@@ -161,39 +161,51 @@ get_itime (time_t current_time)
 }
 
 static void
-update_timeformat(ClockData *cd)
+update_timeformat (ClockData *cd)
 {
-	const char *time;
-	gchar *loc;
+ /* Show date in another line if panel is vertical, or
+  * horizontal but large enough to hold two lines of text
+  */
+#define USE_TWO_LINE_FORMAT(cd) ((cd)->orient == PANEL_APPLET_ORIENT_LEFT  || \
+                                 (cd)->orient == PANEL_APPLET_ORIENT_RIGHT || \
+                                 (cd)->size >= GNOME_Vertigo_PANEL_MEDIUM)
 
-	if (cd->hourformat == 12) {
-		if (cd->showseconds)
-			time = _("%I:%M:%S %p");
-		else
-			time = _("%I:%M %p");
-	} else {
-		if (cd->showseconds)
-			time = _("%H:%M:%S");
-		else
-			time = _("%H:%M");
-	}
+	const char *time_format;
+	const char *date_format;
+	char       *clock_format;
 
-	if (cd->showdate) {
-		/* Show date in another line if panel is vertical, or
-		 * horizontal but large enough to hold two lines of text */
-		if (cd->orient == PANEL_APPLET_ORIENT_LEFT ||
-		    cd->orient == PANEL_APPLET_ORIENT_RIGHT ||
-		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM)
-			loc = g_strconcat (_("%a %b %d"), "\n", time, NULL);
+	if (cd->hourformat == 12)
+		time_format = cd->showseconds ? _("%l:%M:%S %p") : _("%l:%M %p");
+	else
+		time_format = cd->showseconds ? _("%H:%M:%S") : _("%H:%M");
+
+	if (!cd->showdate)
+		clock_format = g_strdup (time_format);
+
+	else {
+		date_format = _("%a %b %d");
+
+		if (USE_TWO_LINE_FORMAT (cd))
+			/* translators: reverse the order of these arguments
+			 *              if the time should come before the
+			 *              date on a clock in your locale.
+			 */
+			clock_format = g_strdup_printf (_("%s\n%s"),
+							date_format, time_format);
 		else
-			loc = g_strconcat (_("%a %b %d"), ", ", time, NULL);
-	} else {
-		loc = g_strdup (time);
+			/* translators: reverse the order of these arguments
+			 *              if the time should come before the
+			 *              date on a clock in your locale.
+			 */
+			clock_format = g_strdup_printf (_("%s, %s"),
+							date_format, time_format);
 	}
 
 	g_free (cd->timeformat);
-	cd->timeformat = g_locale_from_utf8 (loc, -1, NULL, NULL, NULL);
-	g_free (loc);
+	cd->timeformat = g_locale_from_utf8 (clock_format, -1, NULL, NULL, NULL);
+	g_free (clock_format);
+
+#undef USE_TWO_LINE_FORMAT
 }
 
 /* sets accessible name and description for the widget */
@@ -418,7 +430,7 @@ static void
 applet_change_background (PanelApplet               *applet,
 			  PanelAppletBackgroundType  type,
 			  GdkColor                  *color,
-			  const gchar               *pixmap,
+			  GdkPixmap                 *pixmap,
 			  ClockData                 *cd)
 {
 	if (type == PANEL_NO_BACKGROUND) {
@@ -758,7 +770,7 @@ fill_clock_applet(PanelApplet *applet)
 
 	create_clock_widget (cd);
 
-	gtk_container_set_border_width (GTK_CONTAINER (cd->applet), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (cd->applet), 0);
 	gtk_container_add (GTK_CONTAINER (cd->applet), cd->clockw);
 
 	gtk_widget_show (cd->applet);
@@ -882,10 +894,11 @@ properties_response_cb (GtkWidget *widget,
 							      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 		}
 
-		egg_screen_help_display_desktop (
-				gtk_widget_get_screen (cd->applet),
+		egg_help_display_desktop_on_screen (
 				applet_program, "clock",
-				"clock", "clock-settings", &error);
+				"clock", "clock-settings",
+				gtk_widget_get_screen (cd->applet),
+				&error);
 
 		if (error) {
 			GtkWidget *dialog;
@@ -1123,9 +1136,10 @@ display_help_dialog (BonoboUIComponent *uic,
 						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	}
 
-	egg_screen_help_display_desktop (
+	egg_help_display_desktop_on_screen (
+			applet_program, "clock", "clock",NULL,
 			gtk_widget_get_screen (cd->applet),
-			applet_program, "clock", "clock",NULL, &error);
+			&error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
@@ -1188,7 +1202,7 @@ display_about_dialog (BonoboUIComponent *uic,
 		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
 
 	about = gnome_about_new (_("Clock"), VERSION,
-				 _("(C) 1998-2002 the Free Software Foundation"),
+				 "Copyright \xc2\xa9 1998-2002 Free Software Foundation. Inc.",
 				 _("The Clock displays the current time and date"),
 				 authors,
 				 documenters,
